@@ -1,6 +1,6 @@
 ;;;; batch-driver.scm - Driver procedure for the compiler
 ;
-; Copyright (c) 2008-2020, The CHICKEN Team
+; Copyright (c) 2008-2021, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -147,7 +147,7 @@
 		       ((potential-values)
 			(set! pvals (cdar es)))
 		       ((replacable home contains contained-in use-expr closure-size rest-parameter
-				    captured-variables explicit-rest rest-cdr rest-null?)
+				    captured-variables explicit-rest rest-cdr rest-null? consed-rest-arg)
 			(printf "\t~a=~s" (caar es) (cdar es)) )
 		       ((derived-rest-vars)
 			(set! derived-rvars (cdar es)))
@@ -194,8 +194,7 @@
     (register-feature! 'chicken-compile-static))
   (let* ((dynamic (memq 'dynamic options))
 	(unit (memq 'unit options))
-        (initforms `((import-for-syntax ,@default-syntax-imports)
-		     (##core#declare
+	(init-forms `((##core#declare
 		      ,@(append 
 			 default-declarations
 			 (if emit-debug-info
@@ -211,11 +210,12 @@
 				  (or (not compile-module-registration)
 				      (eq? compile-module-registration 'yes)))
 			     '((uses eval-modules))
-			     '())))
-		     ,@(if explicit-use-flag
-			   '()
-			   `((import ,@default-imports)))))
-        (verbose (memq 'verbose options))
+			     '())))))
+	(import-forms `((import-for-syntax ,@default-syntax-imports)
+			,@(if explicit-use-flag
+			      '()
+			      `((import-syntax ,@default-imports)))))
+	(cleanup-forms '(((chicken.base#implicit-exit-handler))))
 	(outfile (cond ((memq 'output-file options) 
 			=> (lambda (node)
 			     (let ((oname (option-arg node)))
@@ -227,14 +227,11 @@
 	(ipath (map chop-separator
 		    (##sys#split-path
 		     (or (get-environment-variable "CHICKEN_INCLUDE_PATH") "")))) 
-
 	(opasses (default-optimization-passes))
 	(time0 #f)
 	(time-breakdown #f)
 	(forms '())
 	(inline-output-file #f)
-	(type-output-file #f)
-	(cleanup-forms '(((chicken.base#implicit-exit-handler))))
 	(profile (or (memq 'profile options)
 		     (memq 'accumulate-profile options) 
 		     (memq 'profile-name options)))
@@ -254,7 +251,7 @@
 	 (and-let* ((m (memq 'module options)))
 	   (option-arg m))))
 
-    (define (cputime) (current-milliseconds))
+    (define (cputime) (current-process-milliseconds))
 
     (define (dribble fstr . args)
       (debugging 'p (apply sprintf fstr args)))
@@ -352,8 +349,9 @@
     (when (memq 'b debugging-chicken) (set! time-breakdown #t))
     (when (memq 'raw options)
       (set! explicit-use-flag #t)
-      (set! cleanup-forms '())
-      (set! initforms '()) )
+      (set! init-forms '())
+      (set! import-forms '())
+      (set! cleanup-forms '()))
     (when (memq 'no-lambda-info options)
       (set! emit-closure-info #f) )
     (when (memq 'no-compiler-syntax options)
@@ -363,7 +361,8 @@
     (when (memq 'inline-global options)
       (set! enable-inline-files #t)
       (set! inline-locally #t))
-    (when verbose
+    (when (memq 'verbose options)
+      (set! verbose-mode #t)
       (set! ##sys#notices-enabled #t))
     (when (memq 'strict-types options)
       (set! strict-variable-types #t)
@@ -392,7 +391,7 @@
       (set! local-definitions #t)
       (set! inline-output-file (option-arg ifile)))
     (and-let* ((tfile (memq 'emit-types-file options)))
-      (set! type-output-file (option-arg tfile)))
+      (set! types-output-file (option-arg tfile)))
     (and-let* ([inlimit (memq 'inline-limit options)])
       (set! inline-max-size 
 	(let ([arg (option-arg inlimit)])
@@ -428,7 +427,6 @@
       (keyword-style #:none)
       (parentheses-synonyms #f)
       (symbol-escape #f) )
-    (set! verbose-mode verbose)
     (set! ##sys#read-error-with-line-number #t)
     (set! ##sys#include-pathnames
       (append (map chop-separator (collect-options 'include-path))
@@ -486,13 +484,19 @@
 		 (lambda (u) (map string->symbol (string-split u ", ")))
 		 (collect-options 'uses))))
       (unless (null? uses)
-	(set! forms
-	  (cons `(##core#declare (uses . ,uses)) forms))))
+	(set! init-forms
+	  (append init-forms `((##core#declare (uses . ,uses)))))))
 
-    ;; Append required extensions to initforms:
-    (set! initforms
+    ;; Mark linked libraries so they will be compiled as unit dependencies.
+    (let ((link (append-map
+		 (lambda (l) (map string->symbol (string-split l ", ")))
+		 (collect-options 'link))))
+      (set! linked-libraries (lset-union/eq? linked-libraries link)))
+
+    ;; Append required extensions to imports:
+    (set! import-forms
       (append
-       initforms
+       import-forms
        (map (lambda (r) `(import ,(string->symbol r)))
 	    (collect-options 'require-extension))))
 
@@ -524,9 +528,9 @@
 	   "you need to specify -profile-name if using accumulated profiling runs"))
 	(set! emit-profile #t)
 	(set! profiled-procedures 'all)
-	(set! initforms
+	(set! init-forms
 	  (append
-	   initforms
+	   init-forms
 	   default-profiling-declarations
 	   (if acc
 	       '((set! ##sys#profile-append-mode #t))
@@ -599,18 +603,23 @@
 	   (print-expr "source" '|1| forms)
 	   (begin-time)
 	   ;; Canonicalize s-expressions
-	   (let* ((exps0 (map (lambda (x)
+	   (let* ((init0 (map canonicalize-expression init-forms))
+		  (exps0 (map (lambda (x)
 				(fluid-let ((##sys#current-source-filename filename))
 				  (canonicalize-expression x)))
-			      (let ((forms (append initforms forms)))
+			      (let ((forms (append import-forms forms)))
 				(if (not module-name)
 				    forms
 				    `((##core#module
 				       ,(string->symbol module-name) ()
 				       ,@forms))))))
+		  (uses0 (map (lambda (u)
+				(canonicalize-expression `(##core#require ,u)))
+			      (##sys#fast-reverse used-libraries)))
 		  (exps (append
 			 (map (lambda (ic) `(set! ,(cdr ic) ',(car ic))) immutable-constants)
-			 (map (lambda (uu) `(##core#callunit ,uu)) used-units)
+			 init0
+			 uses0
 			 (if unit-name `((##core#provide ,unit-name)) '())
 			 (if emit-profile
 			     (profiling-prelude-exps (and (not unit-name)
@@ -629,18 +638,6 @@
 		 (map (lambda (il) (->string (car il)))
 		      import-libraries) ", ")))
 
-	     (and-let* ((reqs (hash-table-ref file-requirements 'dynamic))
-			(missing (remove (cut chicken.load#find-dynamic-extension <> #f) reqs)))
-	       (when (null? (lset-intersection/eq? '(eval repl) used-units))
-		 (notice ; XXX only issued when "-verbose" is used
-		  (sprintf "~A has dynamic requirements but doesn't load (chicken eval): ~A"
-			   (cond (unit-name "unit") (dynamic "library") (else "program"))
-			   (string-intersperse (map ->string reqs) ", "))))
-	       (when (pair? missing)
-		 (warning
-		  (sprintf "the following extensions are not currently installed: ~A"
-			   (string-intersperse (map ->string missing) ", ")))))
-
 	     (when (pair? compiler-syntax-statistics)
 	       (with-debugging-output
 		'S
@@ -679,10 +676,7 @@
 	       (initialize-analysis-database)
 
 	       ;; collect requirements and load inline files
-	       (let* ((req (concatenate (vector->list file-requirements)))
-		      (mreq (concatenate (map cdr req))))
-		 (when (debugging 'M "; requirements:")
-		   (pp req))
+	       (let ((extensions required-libraries))
 		 (when enable-inline-files
 		   (for-each
 		    (lambda (id)
@@ -690,7 +684,7 @@
 					 (symbol->string id) '(".inline") #t #f)))
 			(dribble "Loading inline file ~a ..." ifile)
 			(load-inline-file ifile)))
-		    mreq))
+		    extensions))
 		 (let ((ifs (collect-options 'consult-inline-file)))
 		   (unless (null? ifs)
 		     (set! inline-locally #t)
@@ -717,7 +711,7 @@
 		      (load-type-database
 		       (make-pathname #f (symbol->string id) "types")
 		       enable-specialization))
-		    mreq)
+		    extensions)
 		   (begin-time)
 		   (set! first-analysis #f)
 		   (set! db (analyze 'scrutiny node0))
@@ -764,9 +758,12 @@
 		       (when (memq 'v debugging-chicken)
 			 (dump-global-refs db))
 		       ;; do this here, because we must make sure we have a db
-		       (when type-output-file
-			 (dribble "generating type file `~a' ..." type-output-file)
-			 (emit-types-file filename type-output-file db block-compilation)))
+		       (and-let* ((tfile (or (and (eq? types-output-file #t)
+						  (pathname-replace-extension filename "types"))
+					     (and (string? types-output-file)
+						  types-output-file))))
+			 (dribble "generating type file `~a' ..." tfile)
+			 (emit-types-file filename tfile db block-compilation)))
 		     (set! first-analysis #f)
 		     (end-time "analysis")
 		     (print-db "analysis" '|4| db i)
@@ -853,12 +850,11 @@
 			      (begin-time)
 
                               ;; generate link file
-                              (when emit-link-file
-                                (dribble "generating link file `~a' ..." emit-link-file)
-                                (with-output-to-file
-                                  emit-link-file
-                                  (cut pp linked-static-extensions)))
-                                
+			      (when emit-link-file
+				(let ((exts required-libraries))
+				  (dribble "generating link file `~a' ..." emit-link-file)
+				  (with-output-to-file emit-link-file (cut pp exts))))
+
                                ;; Code generation
 			      (let ((out (if outfile (open-output-file outfile) (current-output-port))) )
 				(dribble "generating `~A' ..." outfile)
