@@ -1,6 +1,6 @@
 ;;;; chicken-status.scm
 ;
-; Copyright (c) 2008-2020, The CHICKEN Team
+; Copyright (c) 2008-2021, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -142,7 +142,9 @@
       ((generated-source-file) (list (list 'generated-source-file mode (cadr info))))
       ((c-include) (list (list 'c-include mode (cadr info))))
       ((scheme-include) (list (list 'scheme-include mode (cadr info))))
-      ((program) (list (list 'program mode (cadr info))))))
+      ((program) (list (list 'program mode (cadr info))))
+      ((c-object) (list (list 'c-object mode (cadr info))))
+      (else '())))
 
   (define (list-installed-components eggs)
     (let ((w (quotient (- (get-terminal-width) 2) 2)))
