@@ -1,6 +1,6 @@
 ;;;; csc.scm - Driver program for the CHICKEN compiler - felix -*- Scheme -*-
 ;
-; Copyright (c) 2008-2020, The CHICKEN Team
+; Copyright (c) 2008-2021, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -68,6 +68,7 @@
 (define osx (eq? (software-version) 'macosx))
 (define cygwin (eq? (software-version) 'cygwin))
 (define aix (eq? (build-platform) 'aix))
+(define solaris (memq (software-version) '(solaris sunos)))
 
 (define elf
   (memq (software-version) '(linux netbsd freebsd solaris openbsd hurd haiku)))
@@ -256,20 +257,21 @@
 (define linking-optimization-options default-linking-optimization-options)
 
 (define link-options '())
+(define rpath-option (if solaris "-R" "-rpath="))
 
 (define (builtin-link-options)
   (append
    (cond (elf
 	  (list
 	   (conc "-L" library-dir)
-	   (conc "-Wl,-R"
+	   (conc "-Wl," rpath-option
 		 (if deployed
 		     "$ORIGIN"
 		     (if host-mode
 			 host-libdir
 			 TARGET_RUN_LIB_HOME)))))
 	 (aix
-	  (list (conc "-Wl,-R\"" library-dir "\"")))
+	  (list (conc "-Wl," rpath-option "\"" library-dir "\"")))
 	 (else
 	  (list (conc "-L" library-dir))))
    (if (and deployed (memq (software-version) '(freebsd openbsd netbsd)))
@@ -671,7 +673,7 @@ EOF
 		(set! compile-options (cons "-DC_EMBEDDED" compile-options)) ]
 	       [(-link)
 		(check s rest)
-		(t-options "-uses" (car rest))
+		(t-options "-link" (car rest))
 		(set! linked-extensions
 		  (append linked-extensions (string-split (car rest) ", ")))
 		(set! rest (cdr rest))]
@@ -776,7 +778,8 @@ EOF
 		(set! rpath (car rest))
 		(when (and (memq (build-platform) '(gnu clang))
 			   (not mingw) (not osx))
-		  (set! link-options (append link-options (list (string-append "-Wl,-R" rpath)))) )
+		  (set! link-options
+                    (append link-options (list (string-append "-Wl," rpath-option rpath)))) )
 	  	(set! rest (cdr rest)) ]
 	       [(-host) #f]
 	       ((-oi) 
@@ -824,11 +827,15 @@ EOF
 			     [(and (> (string-length arg) 3) (string=? "-Wl," (substring arg 0 4)))
 			      (set! link-options (append link-options (list arg))) ]
 			     [(> (string-length arg) 2)
- 			      (let ([opts (cdr (string->list arg))])
-				(if (null? (lset-difference/eq? opts short-options))
- 				    (set! rest
- 				      (append (map (lambda (o) (string-append "-" (string o))) opts) rest) )
- 				    (stop "invalid option `~A'" arg) ) ) ]
+			      (let ([opts (cdr (string->list arg))])
+				(cond ((null? (lset-difference/eq? opts short-options))
+				       (set! rest
+					 (append (map (lambda (o)
+							(string-append "-" (string o))) opts) 
+						 rest) ))
+				      ((char=? #\l (car opts))
+				       (stop "invalid option `~A' - did you mean `-L -l<library>'?" arg))
+				      (else (stop "invalid option `~A'" arg) ) )) ]
 			     [else (stop "invalid option `~A'" s)] ) ]
 		      [(file-exists? arg)
 		       (let-values ([(dirs name ext) (decompose-pathname arg)])
