@@ -1,6 +1,6 @@
 ;;;; egg-info processing and compilation
 ;
-; Copyright (c) 2017-2020, The CHICKEN Team
+; Copyright (c) 2017-2021, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -80,18 +80,29 @@
     ((unix) "cp -r")
     ((windows) "xcopy /y /i /e")))
 
+(define (copy-file-command platform)
+  (case platform
+    ((unix) "cp")
+    ((windows) "copy /y")))
+
 (define (mkdir-command platform)
   (case platform
     ((unix) "mkdir -p")
     ((windows) "mkdir")))
 
 (define (install-executable-command platform)
-  (string-append default-install-program " "
-		 default-install-program-executable-flags))
+  (case platform
+    ((windows) (copy-file-command 'windows))
+    (else
+     (string-append default-install-program " "
+                    default-install-program-executable-flags))))
 
 (define (install-file-command platform)
-  (string-append default-install-program " "
-		 default-install-program-data-flags))
+  (case platform
+    ((windows) (copy-file-command 'windows))
+    (else
+     (string-append default-install-program " "
+                    default-install-program-data-flags))))
 
 (define (remove-file-command platform)
   (case platform
@@ -128,6 +139,19 @@
       (list (implib rtarget))))
 
 
+;;; normalize target path for "random files" (data, c-include, scheme-include)
+
+(define (normalize-destination dest mode)
+  (let ((dest* (normalize-pathname dest)))
+    (if (irregex-search '(: bos ".." ("\\/")) dest*)
+        (error "destination must be relative to CHICKEN install prefix" dest)
+        (normalize-pathname
+         (make-pathname (if (eq? mode 'target)
+                            default-prefix    ; XXX wrong!
+                            (override-prefix "/" host-prefix))
+                        dest*)))))
+
+
 ;;; check condition in conditional clause
 
 (define (check-condition tst mode link)
@@ -264,7 +288,7 @@
                       (dest #f)
                       (files '()))
             (for-each compile-data/include (cddr info))
-            (let* ((dest (or dest 
+            (let* ((dest (or (and dest (normalize-destination dest mode))
                              (if (eq? mode 'target)
                                  default-sharedir    ; XXX wrong!
                                  (override-prefix "/share" host-sharedir))))
@@ -293,8 +317,8 @@
                       (dest #f)
                       (files '()))
             (for-each compile-data/include (cddr info))
-            (let* ((dest (or dest 
-                             (if (eq? mode 'target) 
+            (let* ((dest (or (and dest (normalize-destination dest mode))
+                             (if (eq? mode 'target)
                                  default-incdir   ; XXX wrong!
                                  (override-prefix "/include" host-incdir))))
                    (dest (normalize-pathname (conc dest "/"))))
@@ -308,8 +332,8 @@
                       (dest #f)
                       (files '()))
             (for-each compile-data/include (cddr info))
-            (let* ((dest (or dest
-                             (if (eq? mode 'target) 
+            (let* ((dest (or (and dest (normalize-destination dest mode))
+                             (if (eq? mode 'target)
                                  default-sharedir   ; XXX wrong!
                                  (override-prefix "/share" host-sharedir))))
                    (dest (normalize-pathname (conc dest "/"))))
@@ -513,9 +537,14 @@
                                (if (memq 'static link)
                                    (list (apply compile-static-object data))
                                    '())))))
-                  (else
-                    (let ((data (assq id genfiles)))
-                      (list (apply compile-generated-file data))))))
+                  ((assq id genfiles) =>
+                   (lambda (data)
+                     (list (apply compile-generated-file data))))
+                  ((or (assq id data)
+                       (assq id cinc)
+                       (assq id scminc))
+                   '()) ;; nothing to build for data components
+                  (else (error "Error in chicken-install, don't know how to build component" id))))
           order)
         ;; installation commands
         (append
@@ -940,7 +969,6 @@
                                     output-file)
          srcdir platform)
   (let* ((cmd (install-executable-command platform))
-         (dcmd (remove-file-command platform))
          (mkdir (mkdir-command platform))
          (sname (prefix srcdir name))
          (out (qs* (target-file (conc sname ext) mode) platform #t))
@@ -949,8 +977,6 @@
          (ddir (shell-variable "DESTDIR" platform))
          (destf (qs* (conc dest "/" output-file ext) platform #t)))
     (print "\n" mkdir " " ddir dfile)
-    (when (eq? platform 'unix)
-      (print dcmd " " ddir destf))
     (print cmd " " out " " ddir destf)
     (print-end-command platform)))
 
@@ -1005,7 +1031,6 @@
 
 (define ((install-program name #!key mode output-file) srcdir platform)
   (let* ((cmd (install-executable-command platform))
-         (dcmd (remove-file-command platform))
          (mkdir (mkdir-command platform))
          (ext (executable-extension platform))
          (sname (prefix srcdir name))
@@ -1017,8 +1042,6 @@
          (ddir (shell-variable "DESTDIR" platform))
          (destf (qs* (conc dest "/" output-file ext) platform #t)))
     (print "\n" mkdir " " ddir dfile)
-    (when (eq? platform 'unix)
-      (print dcmd " " ddir destf))
     (print cmd " " out " " ddir destf)
     (print-end-command platform)))
 
@@ -1028,7 +1051,7 @@
          (root (string-append srcdir "/"))
          (mkdir (mkdir-command platform))
          (sfiles (map (cut prefix srcdir <>) files))
-         (dfile (qs* dest platform #t))
+         (dfile (qs* (normalize-destination dest mode) platform #t))
          (ddir (shell-variable "DESTDIR" platform)))
     (print "\n" mkdir " " ddir dfile)
     (let-values (((ds fs) (partition directory? sfiles)))
@@ -1116,16 +1139,16 @@ EOF
     ((windows)
      (printf #<<EOF
 @echo off~%
-set PATH=~a;%PATH%
-set CHICKEN_CC=~a
-set CHICKEN_CXX=~a
-set CHICKEN_CSC=~a
-set CHICKEN_CSI=~a
+set "PATH=~a;%PATH%"
+set "CHICKEN_CC=~a"
+set "CHICKEN_CXX=~a"
+set "CHICKEN_CSC=~a"
+set "CHICKEN_CSI=~a"
 
 EOF
-             (qs* default-bindir platform) (qs* default-cc platform)
-	     (qs* default-cxx platform) (qs* default-csc platform)
-	     (qs* default-csi platform)))))
+             default-bindir default-cc
+	     default-cxx default-csc
+	     default-csi))))
 
 (define ((build-suffix mode name info) platform)
   (case platform
@@ -1227,7 +1250,7 @@ EOF
 
 (define (shell-variable var platform)
   (case platform
-    ((unix) (string-append "${" var "}"))
+    ((unix) (string-append "\"${" var "}\""))
     ((windows) (string-append "%" var "%"))))
 
 ;; NOTE `cmd' must already be quoted for shell
