@@ -1,6 +1,6 @@
 ;;;; egg download
 ;
-; Copyright (c) 2017-2020, The CHICKEN Team
+; Copyright (c) 2017-2021, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -208,7 +208,9 @@
 
 (define (read-chunks in)
   (let get-chunks ((data '()))
-    (let ((size (string->number (read-line in) 16)))
+    (let* ((line (read-line in))
+           (size (and (not (eof-object? line))
+                      (string->number line 16))))
       (cond ((not size)
 	       (error "invalid response from server - please try again"))
             ((zero? size)
