@@ -1,6 +1,6 @@
 ;;;; eval.scm - Interpreter for CHICKEN
 ;
-; Copyright (c) 2008-2020, The CHICKEN Team
+; Copyright (c) 2008-2021, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -591,14 +591,14 @@
 			  (compile `(##sys#provide (##core#quote ,(cadr x))) e #f tf cntr #f)]
 
 			 [(##core#require-for-syntax)
-			  (chicken.load#load-extension (cadr x) '() 'require)
+			  (chicken.load#load-extension (cadr x) #f #f)
 			  (compile '(##core#undefined) e #f tf cntr #f)]
 
 			 [(##core#require)
-			  (let ((id         (cadr x))
-				(alternates (cddr x)))
-			    (let-values (((exp _) (##sys#process-require id #f alternates)))
-			      (compile exp e #f tf cntr #f)))]
+			  (let ((lib (cadr x))
+			  	 (mod (and (pair? (cddr x)) (caddr x))))
+                            (let-values (((reqform _) (##sys#process-require lib mod #f)))
+			      (compile reqform e #f tf cntr #f)))]
 
 			 [(##core#elaborationtimeonly ##core#elaborationtimetoo) ; <- Note this!
 			  (##sys#eval/meta (cadr x))
@@ -607,7 +607,7 @@
 			 [(##core#compiletimetoo)
 			  (compile (cadr x) e #f tf cntr tl?) ]
 
-			 [(##core#compiletimeonly ##core#callunit) 
+			 [(##core#compiletimeonly ##core#callunit ##core#local-specialization)
 			  (compile '(##core#undefined) e #f tf cntr tl?) ]
 
 			 [(##core#declare)
@@ -819,10 +819,11 @@
   (let ((e (##sys#make-structure 'environment 'interaction-environment #f #f)))
     (lambda () e)))
 
-(define-record-printer (environment e p)
-  (##sys#print "#<environment " #f p)
-  (##sys#print (##sys#slot e 1) #f p)
-  (##sys#write-char-0 #\> p))
+(set-record-printer! 'environment
+  (lambda (e p)
+    (##sys#print "#<environment " #f p)
+    (##sys#print (##sys#slot e 1) #f p)
+    (##sys#write-char-0 #\> p)))
 
 (let* ((r4s (chicken.module#module-environment 'r4rs 'scheme-report-environment/4))
        (r5s (chicken.module#module-environment 'scheme 'scheme-report-environment/5))
@@ -933,12 +934,16 @@
 
 (define ##sys#load-dynamic-extension default-load-library-extension)
 
+(define (chicken.load#core-unit? id) ; used by batch-driver.scm
+  (or (memq id core-units)
+      (assq id core-unit-requirements)))
+
 ; these are actually in unit extras, but that is used by default
 
 (define-constant builtin-features
-  '(srfi-30 srfi-46 srfi-61 srfi-62                   ; runtime
-    srfi-0 srfi-2 srfi-8 srfi-9 srfi-11 srfi-15       ; syntax
-    srfi-16 srfi-17 srfi-26 srfi-31 srfi-55 srfi-88)) ; syntax cont
+  '(srfi-30 srfi-46 srfi-61 srfi-62                     ; runtime
+    srfi-0 srfi-2 srfi-8 srfi-9 srfi-11 srfi-15 srfi-16 ; syntax
+    srfi-17 srfi-26 srfi-31 srfi-55 srfi-87 srfi-88))   ; syntax cont
 
 (define default-dynamic-load-libraries
   (case (software-version)
@@ -1121,36 +1126,31 @@
        (##sys#check-list x)
        x) ) ) )
 
-(define load-library/internal
-  (let ((display display))
-    (lambda (uname lib loc)
-      (let ((libs
-	     (if lib
-		 (##sys#list lib)
-		 (cons (##sys#string-append (##sys#slot uname 1) load-library-extension)
-		       (dynamic-load-libraries))))
-	    (top
-	     (c-toplevel uname loc)))
-	(when (load-verbose)
-	  (display "; loading library ")
-	  (display uname)
-	  (display " ...\n") )
-	(let loop ((libs libs))
-	  (cond ((null? libs)
-		 (##sys#error loc "unable to load library" uname _dlerror))
-		((##sys#dload (##sys#make-c-string (##sys#slot libs 0) 'load-library) top))
-		(else
-		 (loop (##sys#slot libs 1)))))))))
+(define (load-unit unit-name lib loc)
+  (unless (##sys#provided? unit-name)
+    (let ((libs
+	   (if lib
+	       (##sys#list lib)
+	       (cons (##sys#string-append (##sys#slot unit-name 1) load-library-extension)
+		     (dynamic-load-libraries))))
+	  (top
+	   (c-toplevel unit-name loc)))
+      (when (load-verbose)
+	(display "; loading library ")
+	(display unit-name)
+	(display " ...\n"))
+      (let loop ((libs libs))
+	(cond ((null? libs)
+	       (##sys#error loc "unable to load library" unit-name (or _dlerror "library not found")))
+	      ((##sys#dload (##sys#make-c-string (##sys#slot libs 0) 'load-library) top)
+	       (##core#undefined))
+	      (else
+	       (loop (##sys#slot libs 1))))))))
 
-(define (##sys#load-library uname #!optional lib loc)
-  (unless (##sys#provided? uname)
-    (load-library/internal uname lib loc)
-    (##core#undefined)))
-
-(define (load-library uname #!optional lib)
-  (##sys#check-symbol uname 'load-library)
+(define (load-library unit-name #!optional lib)
+  (##sys#check-symbol unit-name 'load-library)
   (unless (not lib) (##sys#check-string lib 'load-library))
-  (##sys#load-library uname lib 'load-library))
+  (load-unit unit-name lib 'load-library))
 
 (define ##sys#include-forms-from-file
   (let ((with-input-from-file with-input-from-file)
@@ -1207,25 +1207,29 @@
 		 (or (check pa)
 		     (loop (##sys#slot paths 1)) ) ) ) ) ) ) ))
 
-(define (load-extension/internal id alternates loc)
-  (cond ((##sys#provided? id))
-	((any ##sys#provided? alternates))
-	((memq id core-units)
-         (load-library/internal id #f loc))
-	((find-dynamic-extension id #f) =>
-	 (lambda (ext)
-	   (load/internal ext #f #f #f #f id)
-	   (##sys#provide id)))
-	(else
-	 (##sys#error loc "cannot load extension" id))))
+(define-inline (extension-loaded? lib mod)
+  (cond ((##sys#provided? lib))
+	((eq? mod #t)
+	 (##sys#provided? (module-requirement lib)))
+	((symbol? mod)
+	 (##sys#provided? (module-requirement mod)))
+	(else #f)))
 
-(define (chicken.load#load-extension id alternates loc)
-  (load-extension/internal id alternates loc)
-  (##core#undefined))
+(define (load-extension lib mod loc)
+  (unless (extension-loaded? lib mod)
+    (cond ((memq lib core-units)
+	   (load-unit lib #f loc))
+	  ((find-dynamic-extension lib #f) =>
+	   (lambda (ext)
+	     (load/internal ext #f #f #f #f lib)
+	     (##sys#provide lib)
+	     (##core#undefined)))
+	  (else
+	   (##sys#error loc "cannot load extension" lib)))))
 
 (define (require . ids)
   (for-each (cut ##sys#check-symbol <> 'require) ids)
-  (for-each (cut chicken.load#load-extension <> '() 'require) ids))
+  (for-each (cut load-extension <> #f 'require) ids))
 
 (define (provide . ids)
   (for-each (cut ##sys#check-symbol <> 'provide) ids)
@@ -1235,40 +1239,38 @@
   (for-each (cut ##sys#check-symbol <> 'provided?) ids)
   (every ##sys#provided? ids))
 
+;; Export for internal use in the expansion of `##core#require':
+(define chicken.load#load-unit load-unit)
+(define chicken.load#load-extension load-extension)
+
 ;; Export for internal use in csc, modules and batch-driver:
 (define chicken.load#find-file find-file)
 (define chicken.load#find-dynamic-extension find-dynamic-extension)
 
-;;
-;; Given a library specification, returns three values:
-;;
-;;   - an expression for loading the library, if required
-;;   - a requirement type (e.g. 'dynamic) or #f if provided in core
-;;
-(define (##sys#process-require lib #!optional compiling? (alternates '()) (provided '()) static? mark-static)
-  (let ((id (library-id lib)))
+;; Do the right thing with a `##core#require' form.
+(define (##sys#process-require lib mod compile-mode)
+  (let ((mod (or (eq? lib mod) mod)))
     (cond
-      ((assq id core-unit-requirements) =>
-       (lambda (x) (values (cdr x) #f)))
-      ((memq id builtin-features)
-       (values '(##core#undefined) #f))
-      ((memq id provided)
-       (values '(##core#undefined) #f))
-      ((any (cut memq <> provided) alternates)
-       (values '(##core#undefined) #f))
-      ((memq id core-units)
-       (if compiling?
-	   (values `(##core#declare (uses ,id)) #f)
-	   (values `(##sys#load-library (##core#quote ,id)) #f)))
-      ((and compiling? static?)
-       (mark-static id)
-       (values `(##core#declare (uses ,id)) 'static))
+      ((assq lib core-unit-requirements) => 
+        (lambda (a) (values (cdr a) #t)))
+      ((memq lib builtin-features) 
+        (values '(##core#undefined) #t))
+      ((memq lib core-units)
+        (values
+          (if compile-mode
+   	      `(##core#callunit ,lib)
+	      `(chicken.load#load-unit (##core#quote ,lib)
+	  			       (##core#quote #f)
+				       (##core#quote #f)))
+          #t))
+      ((eq? compile-mode 'static)
+       (values `(##core#callunit ,lib) #f))
       (else
-       (values `(chicken.load#load-extension
-		 (##core#quote ,id)
-		 (##core#quote ,alternates)
-		 (##core#quote require))
-	       'dynamic)))))
+       (values
+          `(chicken.load#load-extension (##core#quote ,lib)
+				        (##core#quote ,mod)
+				        (##core#quote #f))
+          #f)))))
 
 ;;; Find included file:
 
