@@ -1,6 +1,6 @@
 ;;;; modules.scm - module-system support
 ;
-; Copyright (c) 2011-2020, The CHICKEN Team
+; Copyright (c) 2011-2021, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -33,16 +33,18 @@
   (disable-interrupts)
   (fixnum)
   (not inline ##sys#alias-global-hook)
-  (hide check-for-redef find-export find-module/import-library
-	match-functor-argument merge-se module-indirect-exports
-	module-rename register-undefined))
+  (hide check-for-redef compiled-module-dependencies find-export
+	find-module/import-library match-functor-argument merge-se
+	module-indirect-exports module-rename register-undefined))
 
 (import scheme
 	chicken.base
 	chicken.internal
 	chicken.keyword
 	chicken.platform
-	chicken.syntax)
+	chicken.syntax
+	(only chicken.string string-split)
+	(only chicken.format fprintf format))
 
 (include "common-declarations.scm")
 (include "mini-srfi-1.scm")
@@ -311,7 +313,14 @@
 			  (else (hash-table-set! seen (caar se) #t)
 				(lp (cdr se) (cons (car se) se2))))))))))
 
-(define (##sys#compiled-module-registration mod)
+(define (compiled-module-dependencies mod)
+  (let ((libs (filter-map ; extract library names
+	       (lambda (x) (nth-value 1 (##sys#decompose-import x o eq? 'module)))
+	       (module-import-forms mod))))
+    (map (lambda (lib) `(##core#require ,lib))
+	 (delete-duplicates libs eq?))))
+
+(define (##sys#compiled-module-registration mod compile-mode)
   (let ((dlist (module-defined-list mod))
 	(mname (module-name mod))
 	(ifs (module-import-forms mod))
@@ -319,6 +328,9 @@
 	(mifs (module-meta-import-forms mod)))
     `((##sys#with-environment
         (lambda ()
+	  ,@(if (and (eq? compile-mode 'static) (pair? ifs) (pair? sexports))
+		(compiled-module-dependencies mod)
+		'())
           ,@(if (and (pair? ifs) (pair? sexports))
    	        `((scheme#eval '(import-syntax ,@(strip-syntax ifs))))
   	        '())
@@ -373,16 +385,18 @@
 	   'import "cannot find implementation of re-exported syntax"
 	   name))))
   (let* ((sexps
-	  (map (lambda (se)
-		 (if (symbol? se)
-		     (find-reexport se)
-		     (list (car se) #f (##sys#ensure-transformer (cdr se) (car se)))))
-	       sexports))
+	  (filter-map (lambda (se)
+			(and (not (symbol? se))
+			     (list (car se) #f (##sys#ensure-transformer (cdr se) (car se)))))
+		      sexports))
+	 (reexp-sexps
+	  (filter-map (lambda (se) (and (symbol? se) (find-reexport se)))
+		      sexports))
 	 (nexps
 	  (map (lambda (ne)
 		 (list (car ne) #f (##sys#ensure-transformer (cdr ne) (car ne))))
 	       sdefs))
-	 (mod (make-module name lib '() vexports sexps iexports))
+	 (mod (make-module name lib '() vexports (append sexps reexp-sexps) iexports))
 	 (senv (if (or (not (null? sexps))  ; Only macros have an senv
 		       (not (null? nexps))) ; which must be patched up
 		   (merge-se
@@ -450,6 +464,63 @@
       ;; invalid-export: Returns a string if given identifier names a
       ;; non-exportable object. The string names the type (e.g. "an
       ;; inline function"). Returns #f otherwise.
+
+      ;; Given a list of (<identifier> . <source-location>), builds a nicely
+      ;; formatted error message with suggestions where possible.
+      (define (report-unresolved-identifiers unknowns)
+	(let ((out (open-output-string)))
+	  (fprintf out "Module `~a' has unresolved identifiers" (module-name mod))
+
+	  ;; Print filename from a line number entry
+	  (let lp ((locs (apply append (map cdr unknowns))))
+	    (unless (null? locs)
+	      (or (and-let* ((loc (car locs))
+			     (ln (and (pair? loc) (cdr loc)))
+			     (ss (string-split ln ":"))
+			     ((= 2 (length ss))))
+		    (fprintf out "\n  In file `~a':" (car ss))
+		    #t)
+		  (lp (cdr locs)))))
+
+	  (for-each
+	   (lambda (id.locs)
+	     (fprintf out "\n\n  Unknown identifier `~a'" (car id.locs))
+
+	     ;; Print all source locations where this ID occurs
+	     (for-each
+	      (lambda (loc)
+		(define (ln->num ln) (let ((ss (string-split ln ":")))
+				       (if (and (pair? ss) (= 2 (length ss)))
+					   (cadr ss)
+					   ln)))
+		(and-let* ((loc-s
+			    (cond
+			      ((and (pair? loc) (car loc) (cdr loc)) =>
+			       (lambda (ln)
+				 (format "In procedure `~a' on line ~a" (car loc) (ln->num ln))))
+			      ((and (pair? loc) (cdr loc))
+			       (format "On line ~a" (ln->num (cdr loc))))
+			      (else (format "In procedure `~a'" loc)))))
+		  (fprintf out "\n    ~a" loc-s)))
+	      (reverse (cdr id.locs)))
+
+	     ;; Print suggestions from identifier db
+	     (and-let* ((id (car id.locs))
+			(a (getp id '##core#db)))
+	       (fprintf out "\n  Suggestion: try importing ")
+	       (cond
+		 ((= 1 (length a))
+		  (fprintf out "module `~a'" (cadar a)))
+		 (else
+		  (fprintf out "one of these modules:")
+		  (for-each
+		   (lambda (a)
+		     (fprintf out "\n    ~a" (cadr a)))
+		   a)))))
+	   unknowns)
+
+	  (##sys#error (get-output-string out))))
+
       (let* ((explist (module-export-list mod))
 	     (name (module-name mod))
 	     (dlist (module-defined-list mod))
@@ -501,38 +572,16 @@
 							" has not been defined.")))
                                                 (else (bomb "fail")))))))
                               (loop (cdr xl))))))))))
-        (for-each
-	 (lambda (u)
-	   (let* ((where (cdr u))
-		  (u (car u)))
-	     (unless (memq u elist)
-	       (let ((out (open-output-string)))
-		 (set! missing #t)
-		 (display "reference to possibly unbound identifier `" out)
-		 (display u out)
-		 (write-char #\' out)
-		 (when (pair? where)
-		   (display " in:" out)
-		   (for-each
-		    (lambda (sym)
-		      (display "\nWarning:    " out)
-		      (display sym out))
-		    where))
-		 (and-let* ((a (getp u '##core#db)))
-		   (cond ((= 1 (length a))
-			  (display "\nWarning:    suggesting: `(import " out)
-			  (display (cadar a) out)
-			  (display ")'" out))
-			 (else
-			  (display "\nWarning:    suggesting one of:" out)
-			  (for-each
-			   (lambda (a)
-			     (display "\nWarning:    (import " out)
-			     (display (cadr a) out)
-			     (write-char #\) out))
-			   a))))
-		 (##sys#warn (get-output-string out))))))
-	 (reverse (module-undefined-list mod)))
+
+	;; Check all identifiers were resolved
+	(let ((unknowns '()))
+	  (for-each (lambda (u)
+		      (unless (memq (car u) elist)
+			(set! unknowns (cons u unknowns))))
+		    (module-undefined-list mod))
+	  (unless (null? unknowns)
+	    (report-unresolved-identifiers unknowns)))
+
 	(when missing
 	  (##sys#error "module unresolved" name))
 	(let* ((iexports 
@@ -638,7 +687,7 @@
 			     (cond ((null? ids)
 				    (for-each
 				     (lambda (id)
-				       (warn "imported identifier doesn't exist" spec id))
+				       (warn "imported identifier doesn't exist" name id))
 				     missing)
 				    (values name lib `(,head ,spec ,@imports) v s impi))
 				   ((assq (car ids) impv) =>
@@ -773,8 +822,8 @@
        cm
        (merge-se (module-iexports cm) vsi))
       (dm "export-list: " (module-export-list cm)))
-    (import-env (append vsv (import-env)))
-    (macro-env (append vss (macro-env)))))
+    (import-env (merge-se (import-env) vsv))
+    (macro-env (merge-se (macro-env) vss))))
 
 (define (module-rename sym prefix)
   (##sys#string->symbol
@@ -792,8 +841,7 @@
 	       (register-undefined sym mod where))
 	     (module-rename sym (module-name mod))))
 	  (else sym)))
-  (cond ((keyword? sym) sym)
-	((namespaced-symbol? sym) sym)
+  (cond ((namespaced-symbol? sym) sym)
 	((assq sym (##sys#current-environment)) =>
 	 (lambda (a)
 	   (let ((sym2 (cdr a)))
@@ -1071,7 +1119,7 @@
  'srfi-6 'library
  '((get-output-string . chicken.base#get-output-string)
    (open-input-string . chicken.base#open-input-string)
-   (open-output-string . chicken.base#open-input-string)))
+   (open-output-string . chicken.base#open-output-string)))
 
 (##sys#register-primitive-module
  'srfi-8 '() (se-subset '(receive) ##sys#chicken.base-macro-environment))
@@ -1104,8 +1152,11 @@
 (##sys#register-primitive-module
  'srfi-16 '() (se-subset '(case-lambda) ##sys#chicken.base-macro-environment))
 
-(##sys#register-primitive-module
- 'srfi-17 '() (se-subset '(set!) ##sys#default-macro-environment))
+(##sys#register-core-module
+ 'srfi-17 'library
+ '((getter-with-setter . chicken.base#getter-with-setter)
+   (setter . chicken.base#setter))
+ (se-subset '(set!) ##sys#default-macro-environment))
 
 (##sys#register-core-module
  'srfi-23 'library '((error . chicken.base#error)))
