@@ -1,6 +1,6 @@
 ;;;; optimizer.scm - The CHICKEN Scheme compiler (optimizations)
 ;
-; Copyright (c) 2008-2020, The CHICKEN Team
+; Copyright (c) 2008-2021, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -206,6 +206,7 @@
                      ((eq? '##core#variable (node-class arg)))
                      (var (first (node-parameters arg)))
                      ((not (db-get db var 'captured)))
+                     ((not (db-get db var 'consed-rest-arg)))
                      (info (db-get db var 'rest-cdr))
                      (restvar (car info))
                      (depth (cdr info))
@@ -570,6 +571,22 @@
 		       gae)
 		      n2)))))
 
+          ((##core#rest-cdr ##core#rest-car ##core#rest-null? ##core#rest-length)
+	   (let ((rest-var (first params)))
+	     ;; If rest-arg has been replaced with regular arg which
+	     ;; is explicitly consed at call sites, restore rest ops
+	     ;; as regular car/cdr calls on the rest list variable.
+	     ;; This can be improved, as it can actually introduce
+	     ;; many more cdr calls than necessary.
+	     (cond
+              ((or (test rest-var 'consed-rest-arg))
+	       (touch)
+	       (debugging 'o "resetting rest op for explicitly consed rest parameter" rest-var class)
+
+	       (replace-rest-op-with-list-ops class (varnode rest-var) params))
+
+              (else (walk-generic n class params subs fids gae #f))) ) )
+
 	  (else (walk-generic n class params subs fids gae #f)) ) ) )
     
     (define (walk-generic n class params subs fids gae invgae)
@@ -1649,10 +1666,6 @@
 		      (cond [(eq? fnvar (first fnp))
 			     (set! ksites (alist-cons #f n ksites))
 			     (cond [(eq? kvar (first arg0p))
-				    (unless (= argc (length (cdr subs)))
-				      (quit-compiling
-				       "known procedure called recursively with wrong number of arguments: `~A'" 
-				       fnvar) )
 				    (node-class-set! n '##core#recurse)
 				    (node-parameters-set! n (list #t id))
 				    (node-subexpressions-set! n (cddr subs)) ]
@@ -1660,10 +1673,6 @@
 				    => (lambda (a)
 					 (let* ([klam (cdr a)]
 						[kbody (first (node-subexpressions klam))] )
-					   (unless (= argc (length (cdr subs)))
-					     (quit-compiling
-					      "known procedure called recursively with wrong number of arguments: `~A'" 
-					      fnvar) )
 					   (node-class-set! n 'let)
 					   (node-parameters-set! n (take (third (node-parameters klam)) 1))
 					   (node-subexpressions-set!
