@@ -1,6 +1,6 @@
 ;;;; support.scm - Miscellaneous support code for the CHICKEN compiler
 ;
-; Copyright (c) 2008-2020, The CHICKEN Team
+; Copyright (c) 2008-2021, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -34,7 +34,7 @@
      debugging-chicken with-debugging-output quit-compiling
      emit-syntax-trace-info check-signature build-lambda-list
      c-ify-string valid-c-identifier? read-expressions
-     bytes->words words->bytes
+     bytes->words words->bytes replace-rest-op-with-list-ops
      check-and-open-input-file close-checked-input-file fold-inner
      constant? collapsable-literal? immediate? basic-literal?
      canonicalize-begin-body string->expr llist-length llist-match?
@@ -472,8 +472,9 @@
   (parameters node-parameters node-parameters-set!) ; (value...)
   (subexpressions node-subexpressions node-subexpressions-set!)) ; (node...)
 
-(define-record-printer (node n out)
-  (fprintf out "#<node ~a ~a>" (node-class n) (node-parameters n)))
+(set-record-printer! node
+  (lambda (n out)
+    (fprintf out "#<node ~a ~a>" (node-class n) (node-parameters n))))
 
 (define (make-node c p s)
   (##sys#make-structure 'chicken.compiler.support#node c p s))
@@ -778,6 +779,30 @@
 
   (walk node)  )
 
+(define (replace-rest-op-with-list-ops class rest-var-node params)
+  (case class
+    ((##core#rest-car)
+     (make-node '##core#inline
+		(list "C_i_list_ref")
+		(list rest-var-node (qnode (second params)))))
+    ((##core#rest-cdr)
+     (let lp ((cdr-calls (add1 (second params)))
+	      (var rest-var-node))
+       (if (zero? cdr-calls)
+	   var
+	   (lp (sub1 cdr-calls)
+	       (make-node '##core#inline (list "C_i_cdr") (list var))))))
+    ((##core#rest-null?)
+     (make-node '##core#inline
+		(list "C_i_greater_or_equalp")
+		(list (qnode (second params))
+		      (make-node '##core#inline (list "C_i_length") (list rest-var-node)))))
+    ((##core#rest-length)
+     (make-node '##core#inline
+		(list "C_i_length")
+		(list rest-var-node (qnode (second params)))))
+    (else (bomb "Unknown rest op node class while undoing rest op for explicitly consed rest arg. This shouldn't happen!" class))))
+
 ;; Maybe move to scrutinizer.  It's generic enough to keep it here though
 (define (tree-copy t)
   (let rec ([t t])
@@ -840,18 +865,16 @@
 	   (set! lst (cons sym lst))
 	   (set! out (cons (list sym (node->sexpr (cdr val))) out)))))
      db)
-    (if (null? out)
-	(delete-file* inline-file)
-	(with-output-to-file inline-file
-	  (lambda ()
-	    (print "; GENERATED BY CHICKEN " (chicken-version) " FROM "
-		   source-file "\n")
-	    (for-each
-	     (lambda (x)
-	       (pp x)
-	       (newline))
-	     (reverse out))
-	    (print "; END OF FILE"))))
+    (with-output-to-file inline-file
+      (lambda ()
+	(print "; GENERATED BY CHICKEN " (chicken-version) " FROM "
+	       source-file "\n")
+	(for-each
+	 (lambda (x)
+	   (pp x)
+	   (newline))
+	 (reverse out))
+	(print "; END OF FILE")))
     (when (and (pair? lst)
 	       (debugging 'i "the following procedures can be globally inlined:"))
       (for-each (cut print "  " <>) (sort-symbols lst)))))
@@ -1340,17 +1363,17 @@
 	  ((scheme-pointer nonnull-scheme-pointer) '*)
 	  ((blob)
 	   (case mode
-	     ((arg) '(or boolean blob))
+	     ((arg) '(or false blob))
 	     (else 'blob)))
 	  ((nonnull-blob) 'blob)
 	  ((pointer-vector)
 	   (case mode
-	     ((arg) '(or boolean pointer-vector))
+	     ((arg) '(or false pointer-vector))
 	     (else 'pointer-vector)))
 	  ((nonnull-pointer-vector) 'pointer-vector)
 	  ((u8vector u16vector s8vector s16vector u32vector s32vector u64vector s64vector f32vector f64vector)
 	   (case mode
-	     ((arg) `(or boolean (struct ,ft)))
+	     ((arg) `(or false (struct ,ft)))
 	     (else `(struct ,ft))))
 	  ((nonnull-u8vector) '(struct u8vector))
 	  ((nonnull-s8vector) '(struct s8vector))
@@ -1366,10 +1389,15 @@
 		    unsigned-long)
 	   'integer)
 	  ((c-pointer)
-	   '(or boolean pointer locative))
-	  ((nonnull-c-pointer) 'pointer)
+	   (if (eq? 'arg mode)
+	       '(or false pointer locative)
+	       '(or false pointer)))
+	  ((nonnull-c-pointer)
+	   (if (eq? 'arg mode)
+	       '(or pointer locative)
+	       'pointer))
 	  ((c-string c-string* unsigned-c-string unsigned-c-string*)
-	   '(or boolean string))
+	   '(or false string))
 	  ((c-string-list c-string-list*)
 	   '(list-of string))
 	  ((nonnull-c-string nonnull-c-string* nonnull-unsigned-c-string*) 'string)
@@ -1378,10 +1406,15 @@
 	   (cond ((pair? t)
 		  (case (car t)
 		    ((ref pointer function c-pointer)
-		     '(or boolean pointer locative))
+		     (if (eq? 'arg mode)
+			 '(or false pointer locative)
+			 '(or false pointer)))
 		    ((const) (foreign-type->scrutiny-type (cadr t) mode))
 		    ((enum) 'integer)
-		    ((nonnull-pointer nonnull-c-pointer) 'pointer)
+		    ((nonnull-pointer nonnull-c-pointer)
+		     (if (eq? 'arg mode)
+			 '(or pointer locative)
+			 'pointer))
 		    (else '*)))
 		 (else '*)))))))
 
@@ -1916,7 +1949,6 @@ Available debugging options:
      x          display information about experimental features
      D          when printing nodes, use node-tree output
      I          show inferred type information for unexported globals
-     M          show syntax-/runtime-requirements
      N          show the real-name mapping table
      P          show expressions after specialization
      S          show applications of compiler syntax
