@@ -207,6 +207,8 @@
 (define-in-indexed in-s16vector in-s16vector-reverse s16vector-length s16vector-ref)
 (define-in-indexed in-u32vector in-u32vector-reverse u32vector-length u32vector-ref)
 (define-in-indexed in-s32vector in-s32vector-reverse s32vector-length s32vector-ref)
+(define-in-indexed in-u64vector in-u64vector-reverse u64vector-length u64vector-ref)
+(define-in-indexed in-s64vector in-s64vector-reverse s64vector-length s64vector-ref)
 (define-in-indexed in-f32vector in-f32vector-reverse f32vector-length f32vector-ref)
 (define-in-indexed in-f64vector in-f64vector-reverse f64vector-length f64vector-ref)
 
@@ -323,22 +325,23 @@
   (syntax-rules ()
     ((in-random ((var) ()) next . rest) ; XXXX consider in-random-real
      (next ((MAX_RAND (+ (expt 2 29) (- (expt 2 29) 1))))
-           ((var (/ (random MAX_RAND) MAX_RAND)
-                 (/ (random MAX_RAND) MAX_RAND)))
+           ((var (/ (pseudo-random-integer MAX_RAND) MAX_RAND)
+                 (/ (pseudo-random-integer MAX_RAND) MAX_RAND)))
            ()
            ()
            . rest))
     ((in-random ((var) (n)) next . rest)
      (next ((tmp-n n))
-           ((var (random tmp-n) (random tmp-n)))
+           ((var (pseudo-random-integer tmp-n)
+                 (pseudo-random-integer tmp-n)))
            ()
            ()
            ()
         . rest))
     ((in-random ((var) (n lo)) next . rest)
      (next ((tmp-n n) (tmp-lo lo))
-           ((var (+ tmp-lo (random tmp-n))
-                 (+ tmp-lo (random tmp-n))))
+           ((var (+ tmp-lo (pseudo-random-integer tmp-n))
+                 (+ tmp-lo (pseudo-random-integer tmp-n))))
            ()
            ()
            ()
@@ -355,8 +358,8 @@
                        (list->vector tmp-source)
                        tmp-source))
             (tmp-len (vector-length tmp-vec)))
-           ((var (vector-ref tmp-vec (random tmp-len))
-                 (vector-ref tmp-vec (random tmp-len))))
+           ((var (vector-ref tmp-vec (pseudo-random-integer tmp-len))
+                 (vector-ref tmp-vec (pseudo-random-integer tmp-len))))
            ()
            ()
            ()
@@ -479,7 +482,7 @@
       . rest))
     ))
 
-;; Chicken-specific implementation using internal knowledge of the
+;; CHICKEN-specific implementation using internal knowledge of the
 ;; vector+alist representation.  The ##sys#slot form will cause most
 ;; other implementations to choke, so comment this out if needed.
 
