@@ -1,7 +1,8 @@
 ;;;; pp-test.scm
 
 (import (only chicken.pretty-print pp)
-        (only chicken.port with-output-to-string))
+        (only chicken.port with-output-to-string)
+	(only (chicken memory representation) block-ref))
 
 (define (pp->string thing)
   (with-output-to-string (cut pp thing)))
@@ -17,3 +18,4 @@
 (test "\"\\\"\\\"\\\"\"\n" (pp->string "\"\"\""))
 (test "\"\\n\\t\\r\\b\\a\\v\\f\"\n" (pp->string "\n\t\r\b\a\v\f"))
 (test "\\" "\\")                        ; XXX?
+(test "#<unbound value>\n" (pp->string (block-ref 'aardvark 0))) ;; Shouldn't crash
