@@ -1,6 +1,5 @@
 ;;;; module importing from module that reexports core binding
 
 (module foo ()
-  (import scheme chicken)
-  (use reexport-m1)
-  (print (cons* 1 2 3)))
+  (import scheme (chicken base) reexport-m1)
+  (print (cons 1 2)))
