@@ -141,6 +141,11 @@ $compile scrutiny-tests-3.scm -specialize -block
 $compile scrutiny-tests-strict.scm -strict-types -specialize
 ./a.out
 
+echo "======================================== line number database ..."
+$compile -O3 test-line-numbers.scm 2> test-line-numbers.out
+diff $DIFF_OPTS test-line-numbers.expected test-line-numbers.out
+./a.out
+
 echo "======================================== specialization tests ..."
 rm -f foo.types foo.import.*
 $compile specialization-test-1.scm -emit-types-file foo.types -specialize \
@@ -257,6 +262,10 @@ $compile_s reexport-m3.scm -J
 $compile_s reexport-m4.scm -J
 $compile_s reexport-m5.scm -J
 $compile_s reexport-m6.scm -J
+$compile_s reexport-m7.scm -J
+$compile_s reexport-m8.scm -J
+$compile_s reexport-m9.scm -J
+$compile_s reexport-m10.scm -J
 $compile reexport-tests-2.scm
 ./a.out
 
@@ -362,6 +371,9 @@ $compile_r -static module-static-link.scm -o a.out
 echo "======================================== port tests ..."
 $interpret -s port-tests.scm
 
+echo "======================================== read-lines tests ..."
+$interpret -s read-lines-tests.scm
+
 echo "======================================== fixnum tests ..."
 $compile fixnum-tests.scm
 ./a.out
@@ -523,4 +535,4 @@ $interpret -s multiple-values.scm
 $compile multiple-values.scm
 ./a.out
 
-echo "======================================== done."
+echo "======================================== done. All tests passed."
