@@ -11,17 +11,17 @@
 
 ;;;
 
-(let ((p '(1 . 2))
-      (l (list))
-      (n '())
-      (i 123)
-      (f 12.3)
-      (u (+ i f)))
+(let* ((p '(1 . 2))
+       (l (list))
+       (n '())
+       (i 123)
+       (f 12.3)
+       (u (+ i f)))
   (predicate pair? (p) (l n i f))
-  (predicate list? (l n p) (i f))
+  (predicate list? (l n) (p i f))
   (predicate null? (n l) (p i f))
   (predicate fixnum? (i) (f u))
   (predicate exact? (i) (f u))
-  (predicate flonum? (f) (i u))
-  (predicate inexact? (f) (i u))
+  (predicate flonum? (f u) (i))
+  (predicate inexact? (f u) (i))
   (predicate number? (i f u) (n)))
