@@ -1,11 +1,48 @@
 ;;;: test-irregex.scm
 
 
-(import (only chicken.string string-split string-intersperse)
+(import (only chicken.string string-split)
+	(rename (only chicken.string string-intersperse) (string-intersperse string-join)) ;; Avoid srfi-13
         chicken.format chicken.io chicken.irregex chicken.port)
 
 (include "test.scm")
 
+(define (cat . args)
+  (let ((out (open-output-string)))
+    (for-each (lambda (x) (display x out)) args)
+    (get-output-string out)))
+
+(define (warning . args)
+  (for-each (lambda (x) (display x (current-error-port))) args)
+  (newline (current-error-port)))
+
+(define (call-with-input-file file proc)
+  (let* ((in (open-input-file file))
+         (res (proc in)))
+    (close-input-port in)
+    res))
+
+(define (call-with-input-string str proc)
+  (let* ((in (open-input-string str))
+         (res (proc in)))
+    (close-input-port in)
+    res))
+
+(define (call-with-output-string proc)
+  (let ((out (open-output-string)))
+    (proc out)
+    (let ((res (get-output-string out)))
+      (close-output-port out)
+      res)))
+
+(define (port-for-each proc read . o)
+  (let ((in (if (pair? o) (car o) (current-input-port))))
+    (let lp ()
+      (let ((x (read in)))
+        (unless (eof-object? x)
+          (proc x)
+          (lp))))))
+
 (define (subst-matches matches subst)
   (define (submatch n)
     (if (irregex-match-data? matches)
@@ -47,7 +84,7 @@
     (if (list? splt)
 	(apply
 	 (lambda (pattern input result subst output)
-	   (let ((name (sprintf "~A  ~A  ~A  ~A" pattern input result subst)))
+	   (let ((name (cat pattern "  " input "  " result "  " subst)))
 	     (cond
 	      ((equal? "c" result)
 	       (test-error name (matcher pattern input)))
@@ -66,15 +103,16 @@
 
 (for-each
  (lambda (opts)
-   (test-group (sprintf "irregex - ~S" opts)
-     (with-input-from-file "re-tests.txt"
-       (lambda ()
+   (test-group (cat "irregex - " opts)
+     (call-with-input-file "re-tests.txt"
+       (lambda (in)
          (port-for-each
           (lambda (line)
             (test-re (lambda (pat str)
                        (irregex-search (apply irregex pat opts) str))
                      line))
-          read-line)))))
+          read-line
+          in)))))
  '((backtrack)
    (fast)
    ))
@@ -97,7 +135,7 @@
          (let lp ((src (cdr src1))
                   (res (list (substring (caar src1) i (caddar src1)))))
            (if (eq? src src2)
-               (string-intersperse
+               (string-join
                 (reverse (cons (substring (caar src2) (cadar src2) j) res))
                 "")
                (lp (cdr src)
@@ -153,10 +191,9 @@
 
 (for-each
  (lambda (opts)
-   (test-group
-    (sprintf "irregex/chunked - ~S" opts)
-    (with-input-from-file "re-tests.txt"
-      (lambda ()
+   (test-group (cat "irregex/chunked - " opts)
+    (call-with-input-file "re-tests.txt"
+      (lambda (in)
 	(port-for-each
 	 (lambda (line)
 	   (let ((splt (string-split line "\t" #t)))
@@ -164,7 +201,7 @@
 		 (apply 
 		  (lambda (pattern input result subst output)
 		    (let ((name
-			   (sprintf "~A  ~A  ~A  ~A" pattern input result subst)))
+			   (cat pattern "  " input "  " result "  " subst)))
 		      (cond
 		       ((equal? "c" result))
 		       ((equal? "n" result)
@@ -189,7 +226,8 @@
 				 (make-shared-ropes input)))))))
 		  splt)
 		 (warning "invalid regex test line" line))))
-	 read-line)))))
+	 read-line
+	 in)))))
  '((backtrack)
    (fast)
    ))
@@ -198,21 +236,23 @@
 ;; pregexp
 
 '(test-group "pregexp"
-   (with-input-from-file "re-tests.txt"
-     (lambda ()
+   (call-with-input-file "re-tests.txt"
+     (lambda (in)
        (port-for-each
         (lambda (line) (test-re pregexp-match line))
-        read-line))))
+        read-line
+        in))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; default regex (PCRE)
 
 '(test-group "regex"
-   (with-input-from-file "re-tests.txt"
-     (lambda ()
+   (call-with-input-file "re-tests.txt"
+     (lambda (in)
        (port-for-each
         (lambda (line) (test-re string-search line))
-        read-line))))
+        read-line
+        in))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -360,7 +400,7 @@
        (rope "bob@test.com and fred@example.com")
        (lambda (src i s) (reverse s))))
   (test-equal '("poo poo ")
-      (irregex-fold '(* "poo ")
+      (irregex-fold '(+ "poo ")
                     (lambda (i m s)
                       (if (< i (irregex-match-end-index m 0))
                           (cons (irregex-match-substring m) s)
@@ -395,6 +435,12 @@
       (irregex-replace/all 'bol "Line 1\nLine 2" "*"))
   (test-equal "**p*l*a*t*t*e*r"
       (irregex-replace/all '(* "poo ") "poo poo platter" "*"))
+  (test-equal "x- y- z-"
+      (irregex-replace/all '(: (look-behind (or "x" "y" "z")) "a")
+                           "xa ya za"  "-"))
+  (test-equal "any gosh darned string"
+	(irregex-replace/all '(: bos (* whitespace))
+                             "any gosh darned string"  ""))
   (test-equal '("foo" " " "foo" " " "b" "a" "r" " " "foo")
       (irregex-extract '(or (: bow "foo" eow) any) "foo foo bar foo"))
   (test-equal '("f" "o" "o" "b" "a" "r" "b" "a" "z")
@@ -405,6 +451,10 @@
       (irregex-extract (irregex "[aeiou]*") "foobarbaz"))
   (test-equal '("Line 1\n" "Line 2\n" "Line 3")
       (irregex-split 'bol "Line 1\nLine 2\nLine 3"))
+  (test-equal '("foo\n" "bar\n" "baz\n")
+      (irregex-extract '(: bol (+ alpha) newline) "\nfoo\nbar\nbaz\n"))
+  (test-equal '("\nblah" "\nblah" "\nblah")
+      (irregex-extract '(: newline "blah" eol) "\nblah\nblah\nblah\n"))
   )
 
 
@@ -521,6 +571,15 @@
   ;; irregex-flags, irregex-lengths
   )
 
+(test-group "SRE representation edge cases"
+  ;; NFA compilation skipped alternative after empty sequence (#26, found by John Clements)
+  (test-equal "empty sequence in \"or\""
+              ""
+              (irregex-match-substring (irregex-search `(or (seq) "foo") "")))
+  (test-equal "alternative to empty sequence in \"or\""
+              "foo"
+              (irregex-match-substring (irregex-search `(or (seq) "foo") "foo"))))
+
 (test-end)
 
 
